/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.convert;

import java.math.BigDecimal;
import java.math.BigInteger;

import woolpack.fn.Fn;

/**
 * 入力を{@link Number}にキャストして
 * {@link BigDecimal}に変換する{@link Fn}です。
 * このクラスは LSP(The Liskov Substitution Principle) を満たしません。
 * @author nakamura
 *
 * @param <E>
 */
public class ToBigDecimalConverter<E extends Exception> implements Fn<Object, BigDecimal, E> {
	/**
	 * @throws ClassCastException 引数が{@link Number}でない場合。
	 */
	public BigDecimal exec(final Object c) {
		if (c instanceof BigDecimal) {
			return (BigDecimal) c;
		} else if (c instanceof BigInteger) {
			return new BigDecimal((BigInteger) c);
		} else {
			return new BigDecimal(((Number) c).doubleValue());
		}
	}
}
