/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.html;

import woolpack.el.EL;
import woolpack.fn.Fn;
import woolpack.xml.NodeContext;

public class BranchPropertyCounter<C extends NodeContext, E extends Exception> extends AbstractCountProperty<C, E> {
	private Fn<? super C, Void, ? extends E> firstFn;
	private Fn<? super C, Void, ? extends E> pluralFn;

	public BranchPropertyCounter(
			final EL el,
			final Iterable<String> attrNames,
			final Fn<? super C, Void, ? extends E> firstFn,
			final Fn<? super C, Void, ? extends E> pluralFn) {
		super(el, attrNames);
		this.firstFn = firstFn;
		this.pluralFn = pluralFn;
	}

	@Override
	public void interpret(final C context, final PropertyCounter counter) throws E {
		if (counter.getCount() == 1) {
			firstFn.exec(context);
		} else {
			pluralFn.exec(context);
		}
	}

	public Fn<? super C, Void, ? extends E> getFirstFn() {
		return firstFn;
	}
	public void setFirstFn(final Fn<? super C, Void, ? extends E> firstFn) {
		this.firstFn = firstFn;
	}
	public Fn<? super C, Void, ? extends E> getPluralFn() {
		return pluralFn;
	}
	public void setPluralFn(final Fn<? super C, Void, ? extends E> pluralFn) {
		this.pluralFn = pluralFn;
	}
}
