/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.html;

import java.util.Map;
import java.util.Map.Entry;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import woolpack.bool.BoolUtils;
import woolpack.el.GettingEL;
import woolpack.fn.Fn;
import woolpack.fn.FnUtils;
import woolpack.xml.NodeContext;
import woolpack.xml.NodeFindUtils;
import woolpack.xml.XmlUtils;

public class SelectRemaker<E extends Exception> implements Fn<NodeContext, Void, E> {
	private static final Fn<Node, Node, RuntimeException> OPTION =
		NodeFindUtils.one(
			FnUtils.join(XmlUtils.GET_NODE_NAME, BoolUtils.checkEquals("OPTION")));
	
	private GettingEL mapEL;

	public SelectRemaker(final GettingEL mapEL) {
		this.mapEL = mapEL;
	}

	public Void exec(final NodeContext context) {
		final Map configMap;
		{
			final Object o = mapEL.getValue(context);
			if (!(o instanceof Map)) {
				return null;
			}
			configMap = (Map) o;
		}

		final Node templateNode = OPTION.exec(context.getNode());
		XmlUtils.removeChildren(templateNode);

		for (final Object entryObject : configMap.entrySet()) {
			final Entry entry = (Entry) entryObject;
			final Node newNode = templateNode.cloneNode(true);
			((Element) newNode).setAttribute("value", entry.getKey()
					.toString());
			XmlUtils.appendText(newNode, entry.getValue().toString());
			templateNode.getParentNode().insertBefore(newNode, templateNode);
		}
		Node nextNode = templateNode.getNextSibling();
		while (nextNode != null) {
			XmlUtils.removeThis(nextNode);
			nextNode = templateNode.getNextSibling();
		}
		XmlUtils.removeThis(templateNode);
		return null;
	}

	public GettingEL getMapEL() {
		return mapEL;
	}
	public void setMapEL(final GettingEL mapEL) {
		this.mapEL = mapEL;
	}
}
